#include "mex.h"
//#include <cmath>
//#include <omp.h>
#include <iostream>

//#include "Tuples.h"
#include "Peeling.h"
#include "Debug.h"

// Compile (at least on Mac OS X)
// mex -output peeling -v -largeArrayDims Peeling.cpp Tuples.cpp main.cpp ...
//   types.cpp -I/usr/local/igl/igl_lib/include  ...
//   -L/usr/local/igl/igl_lib/lib -liglmatlab ...

void mexFunction(int nlhs, mxArray *plhs[], 
    int nrhs, const mxArray *prhs[])
{  

  if (nrhs != 3) mexErrMsgTxt("Expecting 3 input arguments: elements (numElements x (d+1), values (numElements x 1), constrained facets (numFacets x d)");

  int numElements = (int)mxGetM(prhs[0]);
  int d = (int)mxGetN(prhs[0]);
  if (d != 3 && d != 4) mexErrMsgTxt("Dimension d+1 has to be 3 or 4");  

  int tmp;
  tmp = (int)mxGetM(prhs[1]);
  if (tmp != numElements) mexErrMsgTxt("Numbers of rows of first two arguments have to match");
  if (mxGetN(prhs[1]) != 1) mexErrMsgTxt("Second argument has to be column vector");  

  int numFacets = (int)mxGetM(prhs[2]);
  if (mxGetN(prhs[2]) != d-1) mexErrMsgTxt("Third argument has to be numEdges x (d-1) matrix");  

  if (!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]) || !mxIsDouble(prhs[2])) mexErrMsgTxt("All matrices have to be doubles");

  double *pElements = mxGetPr(prhs[0]);
  double *pValues = mxGetPr(prhs[1]);
  double *pFacets = mxGetPr(prhs[2]);

  BoolVector outside;
  if (d == 3)
  {
	  Peeling<2> peeler;
	  peeler.init(numElements, pElements, pValues, numFacets, pFacets);	  
	  //peeler.peel(outside);
	  peeler.peelOneByOne(outside, true);
  }
  else
  {
	  Peeling<3> peeler;
	  peeler.init(numElements, pElements, pValues, numFacets, pFacets);	  
	  peeler.peelOneByOne(outside, true);
  }
  
  //HierClusteringSlow HCS;
  //HCS.init(numNodes, pValues, pWeights, numEdges, pEdges);
  //HCS.clustering(numClusters);

  if(nlhs > 0)
  {
	  plhs[0] = mxCreateDoubleMatrix(numElements,1, mxREAL);
	  double *pRes = mxGetPr(plhs[0]);
	  for (int e=0; e<numElements; e++) pRes[e] = outside[e];	  
  }
}
